/* -*-Mode: C++;-*-
 * $Id: dynarray.h 1.2.1.1.1.8 Sun, 11 Aug 1996 16:28:57 -0700 jmacd $
 * Dynarray.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _DYNARRAY_H_
#define _DYNARRAY_H_

#include <iostream.h>
#include "config.h"

#define generic template<class T, int DefaultSize, bool ZeroTerm>
#define selftype Dynarray<T, DefaultSize, ZeroTerm>
#define member selftype::

generic class Dynarray {
public:
    Dynarray();
    Dynarray(const T, int N = 1);
    Dynarray(const T*, int N = -1);
    Dynarray(int N);
    Dynarray(const selftype&);
    ~Dynarray();

    selftype& append(const T);
    selftype& append(const T*, int N = -1);
    selftype& append(const selftype&);

    selftype& prepend(const T);
    selftype& prepend(const T*, int N = -1);
    selftype& prepend(const selftype&);

    selftype& assign(const T*, int N = -1);
    selftype& assign(const selftype&);

    T& index(int N, const T& val);
    T& index(int N) const;

    selftype& truncate(unsigned int);
    int length() const;
    void expand(int len, const T& val);

    operator const T*() const;
    const T* cast() const;

    class ArrayIterator {
    public:
	ArrayIterator(selftype* a0) { a = a0; len = a0->length(); i = 0; }
	T& operator*() const  { return a->index(i); }
	void next()           { i += 1; }
	bool finished() const { return i >= len; }
    private:
	selftype* a;
	int len, i;
    };

    void sort(int (*compare)(const void* a, const void* b));

private:
    /* this is so Dstring can have writeme() and Dynarray not */
    friend class Dstring;
    void operator=(const selftype&); /* force use of assign() */
    void maybe_expand(int len);
    T *vec;
    int filled, alloc;
};

#undef member
#undef generic
#undef selftype

#endif
