/* -*-Mode: C++;-*-
 * $Id: fileent.h 1.3.1.2.1.12 Thu, 29 Aug 1996 11:37:17 -0700 jmacd $
 * fileent.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _FILEENT_H_
#define _FILEENT_H_

class FileEntry {

public:

    FileEntry();
    ~FileEntry();

    PrBoolError check_working_file();
    PrVoidError initialize_descriptor(const char* projname,
				      RepEntry* rep,
				      bool regist);

    Dstring* descriptor_name() const { return _descriptor_name; }
    Dstring* internal_descriptor() const { return _internal_descriptor; }
    Sexp* file_entry_sexp() const { return _file_entry_sexp; }
    Sexp* file_descriptor() const { return _file_descriptor; }
    Dstring* full_descriptor_name() const { return _full_rep_path; }
    Dstring* descriptor_version_number() const { return _rcs_version_string; }
    Dstring* working_path() const { return _work_path; }
    Dstring* temp_file_path() const { return _temp_path; }
    Dstring* file_mode() { return _file_mode; }
    FileType file_type() const { return _file_type; };
    const Dstring* link_name() const { return _link_name; }
    bool keyword_sub() const { return _keywords; }
    const char* checksum() const { return _checksum; }
    int keyed_length() const { return _keyed_length; }
    const char* last_mod_date() const { return _date; }
    const char* last_mod_user() const { return _user; }
    bool modified() const { return _modified; }
    bool on_command_line() const { return _on_command_line; }
    void update_project_file();

    void append_tag(const char* tag);
    Dstring* file_tags() { return _tags; }

    bool active_modified() const { return _modified && _on_command_line && _file_type == RealFile; }

    NprVoidError stat(); /* correctly stats links, if type is SymLink */

    mode_t stat_mode() const { return _mode; }
    off_t stat_length() const { return _file_size; }
    time_t stat_mtime() const { return _mtime; }
    ino_t stat_inode() const { return _ino; }
    PrVoidError chmod(time_t date) const;

    PrVoidError get_repository_info(RepEntry* rep_entry);
    PrVoidError get_file_sys_info();

    void update_repository(RepEntry* rep_entry);

    void set_keyword_sub(bool b) { _keywords = b; }
    void set_working_path(Dstring* d) { _work_path = d; }
    void set_file_entry_sexp(Sexp* s) { _file_entry_sexp = s; }
    void set_file_descriptor(Sexp* s) { _file_descriptor = s; }
    void set_temp_file_path(Dstring* d) { _temp_path = d; }
    void set_descriptor_name(Dstring* d) { _descriptor_name = d; }
    void set_internal_descriptor(Dstring* d) { _internal_descriptor = d; }
    void set_full_descriptor_name(Dstring *d) { _full_rep_path = d; }
    void set_descriptor_version_number(Dstring *d) { _rcs_version_string = d; }
    void set_file_mode(Dstring *d) { _file_mode = d; }
    void set_file_type(FileType t) { _file_type = t; }
    void set_link_name(Dstring* d) { _link_name = d; }
    void set_date(const char* d) { _date = d; }
    void set_user(const char* d) { _user = d; }
    void set_on_command_line(bool b) { _on_command_line = b; }
    void set_modified(bool b) { _modified = b; }
    void set_checksum(char* c) { strncpy(_checksum, c, 16); }
    void set_keyed_length(int len) { _keyed_length = len; }
    void set_deleted() { _deleted = true; }

private:

    void nullify_descriptor();

    mode_t _mode;
    time_t _mtime;
    uid_t _owner_uid;
    off_t _file_size;
    ino_t _ino;
    long _keyed_length;
    char _checksum[16];
    bool _modified;
    bool _deleted;

    FileType _file_type;
    Sexp *_file_descriptor;    /* sexp from project file */
    Sexp *_file_entry_sexp;    /* sexp from project file */
    bool _keywords;            /* true if this file is having keyword replacement */
    bool _on_command_line;     /* true if this file was selected */
    bool _initialized;         /* true if initialize_descriptor has been called. */

    /* set by a call to get_{filesys,repository}_info() */
    const char *_user;            /* login name or NULL */
    const char *_date;            /* date string or NULL */

    /* set if :symlink when read */
    Dstring *_link_name;       /* link name or NULL */

    /* set when read */
    Dstring *_work_path;       /* name of this file */
    Dstring *_rcs_version_string; /* rcs version dstring used for saving new version
				   * at checkin.  text from project file, for an
				   * empty descriptor, null string */
    Dstring *_internal_descriptor; /* text from project file of version file name,
				    * for an empty descriptor, null string */
    Dstring *_file_mode;        /* text from project file of file mode */
    Dstring *_descriptor_name; /* unslashified portion of internal_descriptor to serve
				* as a repository file name.  for the internal descriptor
				* project/0_\\\".c in the project file, this is 0_\".c,v */
    Dstring *_tags;           /* If non-null, string containing all :tag=tagname entries. */

    /* set by initialize_descriptor */
    Dstring *_temp_path;       /* name of the full path to the repository file less
				* the ,v */
    Dstring *_full_rep_path;   /* name of the full path to the repository file */
};

#endif /* FILEENT_H */
