/* -*-Mode: C++;-*-
 * $Id: prcsdir.h 1.7 Mon, 02 Sep 1996 16:59:57 -0700 jmacd $
 * file.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _FILE_H_
#define _FILE_H_


extern "C" {
#ifdef sgi
#include <sys/dir.h>       /* SGI's BSD directory functions */
#include <sys/dirent.h>
#define dirent direct
#else
#include <dirent.h>        /* the standard dirent includes */
#endif
}

class Dir {
public:
    Dir() :finished(false) { }
    Dir(const char* path) {
	finished = false;
	thisdir = opendir(path);
	full_name.assign(path);
	full_name_len = full_name.length();
	Dir_next();
    }
    ~Dir() { Dir_close(); }
    bool Dir_open(const char* path) {
	Dir_close();
	thisdir = opendir(path);
	full_name.assign(path);
	full_name_len = full_name.length();
	Dir_next();
	return OK();
    }
    /*
     * Returns true if the directory's state is acceptable.
     */
    bool OK() const { return thisdir != NULL; }
    /*
     * Dir_entry --
     *
     *     Returns the current filename.  Will NOT return "." or "..".
     */
    const char* Dir_entry() const { return entry->d_name; }
    const char* Dir_full_entry()
    {
	if(!finished) {
	    full_name.truncate(full_name_len);
	    full_name.append('/');
	    full_name.append(entry->d_name);
	}
	return full_name;
    }
    /*
     * Dir_next --
     *
     *     Advances the directory to the next entry.  Returns false if
     *     there are none left or an error occurs.  Check OK() for error.
     */
    bool Dir_next() {
	const char* name;
	if(thisdir == NULL) {
	    finished = true;
	    return false;
	}

	if(finished)
	    return false;

	if((entry = readdir(thisdir)) == NULL) {
	    finished = true;
	    return false;
	}

	name = Dir_entry();
	if(name[0] == '.' &&
	   (name[1] == '\0' ||
	    (name[1] == '.' && name[2] == '\0')))
	  return Dir_next();
	return true;
    }
    /*
     * Dir_more --
     *
     *     returns true if more remains to be read.
     */
    bool Dir_finished() const { return finished; }

    void Dir_close() {
	if(thisdir) {
	    closedir(thisdir);
	    thisdir = NULL;
	}
    }

    class DirIterator {
    public:
	DirIterator(Dir& d0) { dp = &d0; }
	const char* operator*() const { return dp->Dir_entry(); }
	void next() { dp->Dir_next(); }
	bool finished() const { return dp->Dir_finished(); }
    private:
	Dir* dp;
    };

    class FullDirIterator {
    public:
	FullDirIterator(Dir& d0) { dp = &d0; }
	const char* operator*() const { return dp->Dir_full_entry(); }
	void next() { dp->Dir_next(); }
	bool finished() const { return dp->Dir_finished(); }
    private:
	Dir* dp;
    };

private:
    bool finished;
    DIR* thisdir;
    dirent* entry;
    int full_name_len;
    Dstring full_name;
};

#endif
