/* -*-Mode: C++;-*-
 * $Id: projdesc.h 1.3.1.1.1.2.1.10.1.1.1.5 Sat, 10 Aug 1996 04:14:55 -0700 jmacd $
 * projdesc.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _PROJDESC_H_
#define _PROJDESC_H_

#define foreach_fileent(name, project) \
    foreach(name, (project)->file_entries(), FileEntryPtrArray::ArrayIterator)

PrProjectDescriptorPtrError read_project_file(const char* project_full_name,
					      const char* file_name);
PrProjectDescriptorPtrError read_project_file(const char* project_full_name,
					      const char* file_name,
					      FILE* pipe);
ProjectDescriptor* fake_project_descriptor();

PrSexpPtrError parse_prj_file(FILE* infile, const char* source_name);

class ProjectDescriptor {

public:

    ProjectDescriptor() { }
    ~ProjectDescriptor();

    PrVoidError write_project_file(const char* filename) const;
    void append_file(const char*);
    void delete_file(FileEntry*);

    Dstring* project_version_name() const { return _project_version_name;}
    Dstring* project_version_major() const { return _project_version_major; }
    Dstring* project_version_minor() const { return _project_version_minor; }

    Dstring* parent_version_name() const { return _parent_version_name; }
    Dstring* parent_version_major() const { return _parent_version_major; }
    Dstring* parent_version_minor() const { return _parent_version_minor; }

    Dstring* project_description() const { return _project_description;}
    Dstring* version_log() const { return _version_log; }
    Dstring* new_version_log() const { return _new_version_log; }
    Dstring* checkin_time() const { return _checkin_time; }
    Dstring* checkin_login() const { return _checkin_login; }
    Dstring* created_by_major() const { return _created_by_major; }
    Dstring* created_by_minor() const { return _created_by_minor; }
    Dstring* created_by_micro() const { return _created_by_micro; }

    Dstring* descends_from_name() const { return _descends_from_name; }
    Dstring* descends_from_major() const { return _descends_from_major; }
    Dstring* descends_from_minor() const { return _descends_from_minor; }

    /* merge_continuing instructs the project descriptor that future
     * merge_notify and has_been_merged requests refer to the named
     * project. */
    bool merge_continuing(const char* major, const char* minor);
    void merge_notify_incomplete() { _merge_incomplete = true; }
    void merge_notify_complete()   { _merge_complete = true; }
    void merge_notify(FileEntry* w, FileEntry* c, FileEntry* s, MergeAction ma);
    bool has_been_merged(FileEntry* w, FileEntry* c, FileEntry* s) const;
    void merge_finish();
    PrVoidError adjust_merge_parents();

    Sexp* populate_ignore() const { return _populate_ignore; }

    void read_quick_elim();
    void quick_elim_unmodified();
    void quick_elim_update();
    void quick_elim_update_fe(FileEntry* fe);

    PrVoidError update_repository();

    ProjectVersionData* project_version_data() const { return _project_version_data; }

    FileEntryPtrArray* file_entries() { return _file_entries; }

    PrVoidError init_repository_entry(const char* name, bool lock, bool create);
    PrVoidError resolve_checkin_version(const char* maj);
    RepEntry* repository_entry() const { return _repository_entry; }
    void repository_entry(RepEntry* rep) { _repository_entry = rep; }
    PrVoidError checkin_project_file();
    void set_descends_from(const char* major, int minor);

    const char* project_full_name() const { return _project_full_name; }
    const char* project_name() const { return _project_name; }
    const char* project_file_path() const { return _project_file_path; }
    const char* project_aux_path() const { return _project_aux_path; }
    const char* project_path() const { return _project_path; }

private:

    PrVoidError created_by_prj_entry_func(SexpTable* table);
    PrVoidError project_ver_prj_entry_func(SexpTable* table);
    PrVoidError parent_ver_prj_entry_func(SexpTable* table);
    PrVoidError project_desc_prj_entry_func(SexpTable* table);
    PrVoidError version_log_prj_entry_func(SexpTable* table);
    PrVoidError new_version_log_prj_entry_func(SexpTable* table);
    PrVoidError checkin_time_prj_entry_func(SexpTable* table);
    PrVoidError checkin_login_prj_entry_func(SexpTable* table);
    PrVoidError files_prj_entry_func(SexpTable* table);
    PrVoidError merge_parents_prj_entry_func(SexpTable* table);
    PrVoidError new_merge_parents_prj_entry_func(SexpTable* table);
    PrVoidError descends_from_prj_entry_func(SexpTable* table);
    PrVoidError populate_ignore_prj_entry_func(SexpTable* table);

    PrVoidError verify_merge_parents(Sexp* s);
    PrVoidError strongly_verify_merge_parents(Sexp* s);
    PrVoidError bad_project_file(Sexp* s, const char* message);
    PrVoidError individual_insert_fileent(Sexp* file, FileEntry* entry);

    PrVoidError init_from_file(const char* project_full_name, const char* filename, FILE* infile);

    friend PrProjectDescriptorPtrError read_project_file(const char* project_full_name,
							 const char* filename);

    friend PrProjectDescriptorPtrError read_project_file(const char* project_full_name,
							 const char* file_name,
							 FILE* pipe);

    friend ProjectDescriptor* fake_project_descriptor();

    PrVoidError verify_project_file();
    void init_fake_project();

    ProjectDescriptor(const ProjectDescriptor&); /* illegal */
    operator=(const ProjectDescriptor&); /* illegal */

    QuickElim* _quick_elim;
    Dstring _project_full_name;
    const char* _project_name;
    Dstring _project_file_path;
    Dstring _project_aux_path;
    Dstring _new_rcs_version;
    Dstring _project_path;

    Dstring* last_files_dstring;
    Sexp* substrings;
    Sexp* sexps;

    RepEntry* _repository_entry;
    FileEntryPtrArray* _file_entries;
    ProjectVersionData* _project_version_data;

    Dstring* _project_version_name;
    Dstring* _project_version_major;
    Dstring* _project_version_minor;

    Dstring* _parent_version_name;
    Dstring* _parent_version_major;
    Dstring* _parent_version_minor;

    Dstring* _project_description;
    Dstring* _version_log;
    Dstring* _new_version_log;
    Dstring* _checkin_time;
    Dstring* _checkin_login;

    Dstring* _created_by_major;
    Dstring* _created_by_minor;
    Dstring* _created_by_micro;

    Dstring* _descends_from_name;
    Dstring* _descends_from_major;
    Dstring* _descends_from_minor;

    SexpTable* _merge_data_w_lookup_table;
    SexpTable* _merge_data_c_lookup_table;
    SexpTable* _merge_data_s_lookup_table;
    Dstring* _merge_data_insertion_point;
    Sexp* _merge_parents_prj_entry;
    Sexp* _new_merge_parents_prj_entry;
    Dstring* _merge_selected_major;
    Dstring* _merge_selected_minor;
    bool _merge_complete;
    bool _merge_incomplete;

    Sexp* _populate_ignore;
};
#endif /* PROJDESC_H */
