/* -*-Mode: C++;-*-
 * $Id: rebuild.h 1.9 Sun, 30 Jun 1996 12:25:54 -0700 jmacd $
 * rebuild.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _REBUILD_H_
#define _REBUILD_H_

extern const char prcs_data_file_name[];

PrVoidError admin_rebuild_command_no_open(RepEntry* rep, bool valid_rep_data);

PrRebuildFilePtrError read_repository_data(const char* project, const char* filename0, bool write);

PrVoidError create_empty_data(const char* project, const char* filename0);

class RebuildFile {
public:

    friend PrRebuildFilePtrError read_repository_data(const char* project,
						      const char* filename0,
						      bool write);

    ~RebuildFile();

    ProjectVersionDataPtrArray* get_project_summary() const {
	return project_data; }
    RcsFileTable* get_rcs_file_summary() const { return rcs_file_table; }

    void add_project_data(ProjectVersionData* data);
    void add_rcs_file_data(const char* filename, RcsVersionData* data);

    PrVoidError update_project_data();

private:

    PrVoidError init_from_file(const char*, const char*, bool);

    PrVoidError read_header(const char* projname);
    PrVoidError read_rcs_file_summary();
    PrVoidError read_project_summary();

    PrRcsVersionDataPtrError read_rcs_version();
    PrProjectVersionDataPtrError read_project_version();

    PrVoidError bad_data_file();

    RebuildFile()
	:buf(NULL), os(NULL) { }

    const char* get_string(); /* null if fail */
    const char* get_string(int len);
    /* expect a string, return false if failure */
    bool get_string(const char*, bool term = false);
    int get_size(); /* < 0 if fail */
    const char* source_name(); /* returns filename */
    bool done(); /* true if file has been completely read */
    void init_stream(); /* initialize os for writing */

    int offset;
    Dstring filename;
    const char* last;
    MemorySegment *seg;

    ProjectVersionDataPtrArray* project_data;
    RcsFileTable* rcs_file_table;

    streambuf* buf;
    ostream* os;
};

char* md5_buffer(const char* buffer, int buflen);

#endif
