/* -*-Mode: C++;-*-
 * $Id: setkeys.h 1.3.1.7 Mon, 02 Sep 1996 16:59:57 -0700 jmacd $
 * setkeys.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _SETKEYS_H_
#define _SETKEYS_H_

/*
 * setkeys, setkeys_infile, setkeys_inputbuf --
 *
 *     these three commands differ only in where they get their input.
 *     setkeys takes input from a named file.  setkeys_infile takes
 *     input from a FILE* already opened for reading.  setkeys_inputbuf
 *     takes the address and length of a memory segment containing the
 *     file to be compared.  the first integer argument is to be one of
 *     the symbolic constants SETKEYS or UNSETKEYS.  the second integer
 *     argument is to be one of NOSUPPRESS, SUPPRESSIFNOCHANGE, and
 *     SUPPRESSUNCONDITIONALLY.
 *
 *     The exit code is 0 if the output file does not (or would not)
 *     differ from the input, 1 if the output file does (would) differ,
 *     and 2 in case of trouble.
 */

struct SetkeysData {
    int numkeys;
    const char* const* keypairs;
    bool setkeys;
};

extern SetkeysData Setkeys;
extern SetkeysData Unsetkeys;

PrBoolError setkeys(const char* fromfile,
		    const char* tofile,
		    SetkeysData data);
PrBoolError setkeys_infile(int fd,
			   const char* tofile,
			   SetkeysData data);
PrBoolError setkeys_inputbuf(const char* inputbuf,
			     int inputbuflen,
			     ostream* os,
			     SetkeysData data);
PrBoolError setkeys_inoutbuf(const char* inputbuf,
			     int inputbuflen,
			     MemorySegment *seg,
			     SetkeysData data);
PrBoolError setkeys_outbuf(const char* file,
			   MemorySegment *seg,
			   SetkeysData data);

#endif
