/* -*-Mode: C++;-*-
 * $Id: sexp.h 1.2.1.6 Sun, 30 Jun 1996 04:25:49 -0700 jmacd $
 * sexp.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _SEXP_H_
#define _SEXP_H_

class Sexp {

    enum SexpType { Pair, Symbol };

public:

    Sexp(Dstring* s, int line = -1) {
	_sexp._symbol._key = s;
	_sexp._symbol._next = NULL;
	_sexp._symbol._prev = NULL;
	_linenum = line;
	_type = Symbol; }
    Sexp(Sexp *s1, Sexp *s2, int line = -1) {
	_sexp._pair._pcar = s1;
	_sexp._pair._pcdr = s2;
	_linenum = line;
	_type = Pair; }
    ~Sexp() {
	if(_type == Pair) {
	    if(car()->is_pair())
		delete _sexp._pair._pcar;
	    delete _sexp._pair._pcdr;
	}
    }

    bool is_pair() const { return _type == Pair; }
    Dstring* key() const { return _sexp._symbol._key; }
    Sexp* next() const { return _sexp._symbol._next; }
    Sexp* prev() const { return _sexp._symbol._prev; }
    void next(Sexp* s) { _sexp._symbol._next = s; }
    void prev(Sexp* s) { _sexp._symbol._prev = s; }
    Sexp* car() const { return _sexp._pair._pcar; }
    Sexp* cdr() const { return _sexp._pair._pcdr; }
    Sexp* cadr() const { return cdr()->car(); }
    Sexp* caar() const { return car()->car(); }
    bool is_empty() const { return cdr() == NULL; }
    Dstring* empty_string() const { return car()->key(); }
    int line_number() const { return _linenum; }
    Sexp* setcdr(Sexp* ncdr) { _sexp._pair._pcdr = ncdr; return this; }

    int length() const;
    Sexp* nreverse();

private:
    union {
	struct { Dstring *_key; Sexp *_next; Sexp* _prev; } _symbol;
	struct { Sexp *_pcar; Sexp *_pcdr; } _pair;
    } _sexp;

    SexpType _type;
    int _linenum;
};

extern Sexp* new_empty_list();
extern ostream& operator<<(ostream& os, Sexp *s);

#endif
