/* -*-Mode: C++;-*-
 * $Id: typedefs.h 1.8.1.10.1.2 Mon, 12 Aug 1996 23:59:17 -0700 jmacd $
 * typedefs.h:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#ifndef _TYPEDEFS_H_
#define _TYPEDEFS_H_

enum ErrorToken {
    NonFatalError           = 1 << 0,
    FatalError              = 1 << 1,
    InternalError           = 1 << 2,
    InternalRepositoryError = 1 << 3,
    RepositoryError         = 1 << 4,
    WriteFailure            = 1 << 5,
    ReadFailure             = 1 << 6,
    StatFailure             = 1 << 7,
    UserAbort               = 1 << 8
};

enum MissingFileAction {
    QueryUserRemoveFromCommandLine,
    NoQueryUserRemoveFromCommandLine,
    SetUnmodifiedWarnUser
};

enum OverwriteStatus { DoesntExist, IgnoreMe, SameType };

enum NonErrorToken { NoError };

enum PrcsExitStatus {
    ExitSuccess,
    ExitDiffs,
    ExitNoDiffs
};

enum MergeAction {
    MergeActionMerge   = 'm',
    MergeActionAdd     = 'a',
    MergeActionDelete  = 'd',
    MergeActionNothing = 'n',
    MergeActionReplace = 'r'
};

enum FileType {
    SymLink,
    Directory,
    RealFile
};

#ifdef __GNUG__
#ifndef __REMOVE_ME_WHEN_GPLUSPLUS_STOPS_SUCKING__
/* Forward template declarations don't work in g++ 2.7.x, so we have
 * unneccesary includes here. */
#include "dstring.h"
#include "dynarray.h"
#include "hash.h"
#include "populate.h"
#include "prcserror.h"
#endif
#endif

class PrVoidError;                    /* include/prcserror.h */
template <class Type> class PrError;  /* include/prcserror.h */

#ifdef PRCS_DEVEL
class NprVoidError;                   /* include/prcserror.h */
template <class Type> class NprError; /* include/prcserror.h */
#else
typedef PrVoidError NprVoidError;
#endif

class Dstring;            /* include/dstring.h */
class UpgradeRepository;  /* include/convert.h */
class ProjectDescriptor;  /* include/projdesc.h */
class RepEntry;           /* include/repository.h */
class FileEntry;          /* include/fileent.h */
class AdvisoryLock;       /* include/lock.h */
class PrettyOstream;      /* include/prcserror.h */
class PrettyStreambuf;    /* include/prcserror.h */
class Sexp;               /* include/sexp.h */
class PipeRec;            /* include/syscmd.h */
class SystemCommand;      /* include/syscmd.h */
class RcsDelta;           /* include/vc.h */
class RcsVersionData;     /* include/vc.h */
class ProjectVersionData; /* include/vc.h */
class FileRecord;         /* include/populate.h */
class Dir;                /* include/prcsdir.h */
class MemorySegment;      /* include/memory.h */
class RebuildFile;        /* include/rebuild.h */
class QuickElim;          /* include/quick.h */

template <class T, int DefaultSize, bool ZeroTerm> class Dynarray; /* include/dynarray.h */
template <class Key, class Data> class HashTable; /* include/hash.h */
template <class X, class Y> class Pair; /* include/hash.h */
template <class T> class List;          /* include/hash.h */

typedef Dynarray< char const*, 64, true> ArgList;
typedef Dynarray< char const*, 64, true> CharPtrArray;
typedef Dynarray< RcsVersionData*, 64, false > RcsVersionDataPtrArray;
typedef Dynarray< RcsVersionDataPtrArray*, 64, false > RcsVersionDataPtrArrayPtrArray;
typedef Dynarray< ProjectVersionData*, 64, false > ProjectVersionDataPtrArray;
typedef Dynarray< RcsDelta*, 64, false > RcsDeltaPtrArray;
typedef Dynarray< Dstring*, 64, false > DstringPtrArray;
typedef Dynarray< FileEntry*, 64, false > FileEntryPtrArray;

typedef HashTable< const char* , FileType > PathTable;
typedef HashTable< ino_t , FileType > InoTable;
typedef HashTable< const char*, Sexp* > SexpTable;
typedef HashTable< const char* , FileEntry* > FileTable;
typedef HashTable< const char*, RcsDelta* > RcsFileTable;

typedef List< FileRecord > FileRecordList;

typedef PrError< ArgList* > PrArgListPtrError;
typedef PrError< int > PrExitStatusError;
typedef PrError< int > PrIntError;
typedef PrError< PrcsExitStatus > PrPrcsExitStatusError;
typedef PrError< time_t > PrTimeTError;
typedef PrError< bool > PrBoolError;
typedef PrError< const char* > PrConstCharPtrError;
typedef PrError< Dstring* > PrDstringPtrError;
typedef PrError< ProjectDescriptor* > PrProjectDescriptorPtrError;
typedef PrError< RepEntry* > PrRepEntryPtrError;
typedef PrError< PipeRec* > PrPipeRecPtrError;
typedef PrError< FILE* > PrCFilePtrError;
typedef PrError< Sexp* > PrSexpPtrError;
typedef PrError< RcsVersionData* > PrRcsVersionDataPtrError;
typedef PrError< ProjectVersionData* > PrProjectVersionDataPtrError;
typedef PrError< RcsDelta* > PrRcsDeltaPtrError;
typedef PrError< RcsVersionDataPtrArray* > PrRcsVersionDataPtrArrayPtrError;
typedef PrError< ProjectVersionDataPtrArray* > PrProjectVersionDataPtrArrayPtrError;
typedef PrError< RcsDeltaPtrArray* > PrRcsDeltaPtrArrayPtrError;
typedef PrError< RcsFileTable* > PrRcsFileTablePtrError;
typedef PrError< QuickElim* > PrQuickElimPtrError;
typedef PrError< RebuildFile* > PrRebuildFilePtrError;
typedef PrError< OverwriteStatus > PrOverwriteStatusError;
typedef PrError< FILE* > PrCFilePtrError;

#ifdef PRCS_DEVEL
typedef NprError< ArgList* > NprArgListPtrError;
typedef NprError< int > NprExitStatusError;
typedef NprError< int > NprIntError;
typedef NprError< PrcsExitStatus > NprPrcsExitStatusError;
typedef NprError< time_t > NprTimeTError;
typedef NprError< bool > NprBoolError;
typedef NprError< const char* > NprConstCharPtrError;
typedef NprError< Dstring* > NprDstringPtrError;
typedef NprError< ProjectDescriptor* > NprProjectDescriptorPtrError;
typedef NprError< RepEntry* > NprRepEntryPtrError;
typedef NprError< PipeRec* > NprPipeRecPtrError;
typedef NprError< FILE* > NprCFilePtrError;
typedef NprError< Sexp* > NprSexpPtrError;
typedef NprError< RcsVersionData* > NprRcsVersionDataPtrError;
typedef NprError< ProjectVersionData* > NprProjectVersionDataPtrError;
typedef NprError< RcsDelta* > NprRcsDeltaPtrError;
typedef NprError< RcsVersionDataPtrArray* > NprRcsVersionDataPtrArrayPtrError;
typedef NprError< ProjectVersionDataPtrArray* > NprProjectVersionDataPtrArrayPtrError;
typedef NprError< RcsDeltaPtrArray* > NprRcsDeltaPtrArrayPtrError;
typedef NprError< RcsFileTable* > NprRcsFileTablePtrError;
typedef NprError< QuickElim* > NprQuickElimPtrError;
typedef NprError< RebuildFile* > NprRebuildFilePtrError;
typedef NprError< OverwriteStatus > NprOverwriteStatusError;
typedef NprError< FILE* > NprCFilePtrError;
typedef NprError< char > NprCharError;
#else
typedef PrArgListPtrError NprArgListPtrError;
typedef PrExitStatusError NprExitStatusError;
typedef PrIntError NprIntError;
typedef PrPrcsExitStatusError NprPrcsExitStatusError;
typedef PrTimeTError NprTimeTError;
typedef PrBoolError NprBoolError;
typedef PrConstCharPtrError NprConstCharPtrError;
typedef PrDstringPtrError NprDstringPtrError;
typedef PrProjectDescriptorPtrError NprProjectDescriptorPtrError;
typedef PrRepEntryPtrError NprRepEntryPtrError;
typedef PrPipeRecPtrError NprPipeRecPtrError;
typedef PrCFilePtrError NprCFilePtrError;
typedef PrSexpPtrError NprSexpPtrError;
typedef PrRcsVersionDataPtrError NprRcsVersionDataPtrError;
typedef PrProjectVersionDataPtrError NprProjectVersionDataPtrError;
typedef PrRcsDeltaPtrError NprRcsDeltaPtrError;
typedef PrRcsVersionDataPtrArrayPtrError NprRcsVersionDataPtrArrayPtrError;
typedef PrProjectVersionDataPtrArrayPtrError NprProjectVersionDataPtrArrayPtrError;
typedef PrRcsDeltaPtrArrayPtrError NprRcsDeltaPtrArrayPtrError;
typedef PrRcsFileTablePtrError NprRcsFileTablePtrError;
typedef PrQuickElimPtrError NprQuickElimPtrError;
typedef PrRebuildFilePtrError NprRebuildFilePtrError;
typedef PrOverwriteStatusError NprOverwriteStatusError;
typedef PrCFilePtrError NprCFilePtrError;
#endif

#endif
