/* -*-Mode: C++;-*-
 * $Id: rekey.cc 1.4.1.3 Mon, 02 Sep 1996 16:59:57 -0700 jmacd $
 * rekey.cc:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#include "prcs.h"
#include "checkout.h"
#include "checkin.h"
#include "projdesc.h"
#include "fileent.h"
#include "repository.h"
#include "misc.h"
#include "memseg.h"
#include "setkeys.h"
#include "system.h"

static PrBoolError rekey_file(FileEntry* fe, ProjectDescriptor* project);

PrPrcsExitStatusError rekey_command()
{
    ProjectDescriptor *project;

    Return_if_fail(project << read_project_file(cmd_root_project_full_name,
						cmd_root_project_file_path));

    Return_if_fail(project->init_repository_entry(cmd_root_project_name, false, false));

    eliminate_unnamed_files(project);

    Return_if_fail(eliminate_working_files(project, NoQueryUserRemoveFromCommandLine));

    Return_if_fail(warn_unused_files(true));

    prcsinfo << "Setting keys in project " << squote(cmd_root_project_name)
	     << " for version " << project->project_version_major() << '.'
	     << project->project_version_minor() << dotendl;

    int processed = 0;

    foreach_fileent(fe_ptr, project) {
	FileEntry* fe = *fe_ptr;
	bool replaced_keys;

	if(fe->file_type() != RealFile ||
	   !fe->keyword_sub() ||
	   !fe->on_command_line() ||
	   !fe->descriptor_name())
	    continue;

	Return_if_fail(replaced_keys << rekey_file(fe, project));

	if(replaced_keys)
	    processed += 1;
    }

    if(option_long_format && !option_report_actions)
	prcsoutput << processed << " files rekeyed" << dotendl;

    return ExitSuccess;
}

static MemorySegment setkeys_buffer;

static PrBoolError rekey_file(FileEntry* fe, ProjectDescriptor* project)
{
    RepEntry* rep_entry = project->repository_entry();
    RcsVersionData* version_data;
    bool changed;
    const char* filename = *fe->working_path();

    Return_if_fail(fe->get_repository_info(rep_entry));

    Return_if_fail(version_data << rep_entry->lookup_rcs_file_data(*fe->descriptor_name(),
								   *fe->descriptor_version_number()));

    make_keyword_pair_list(fe, project);

    Return_if_fail(changed << setkeys_outbuf(filename,
					     &setkeys_buffer,
					     Setkeys));

    if(!option_report_actions && changed) {

	WriteableFile outfile;

	Return_if_fail(outfile.open(filename));

	Return_if_fail(outfile.write(setkeys_buffer.segment(), setkeys_buffer.length()));

	Return_if_fail(outfile.close());
    }

    if(option_long_format && changed) {
	prcsinfo << (option_report_actions ? "Replace" : "Replaced")
		 << " keywords in file " << squote(fe->working_path()) << dotendl;
    }

    return changed;
}
