/* -*-Mode: C++;-*-
 * $Id: sexp.cc 1.2.1.5 Fri, 14 Jun 1996 02:43:49 -0700 jmacd $
 * sexp.cc:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Author: Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#include "prcs.h"
#include "sexp.h"

static void print_sexp(const Sexp* S, ostream& os);

/* warning: this file uses a somewhat abnormal convention so you won't
   want to use it without reading it carefully.  the convention is that
   empty lists are not sexps with a NULL cdr, rather, they have an empty
   Dstring and one more sexp as cdr, so to test for an empty list, use
   is_empty or check if cdr() == NULL.  when constructing the lists in
   empty strings are added at the ends, this is why */

/* also: results of car() and cdr() and the other union accessors are
   going to cause an error, so check for is_pair() before calling car()
   or cdr() or setcdr() or whatever */

#if 0
Sexp::Sexp(Dstring* s, int line)
{
    thesexp.key = s;
    type = 0;
    linenum = line;
}

Sexp::Sexp(Sexp *s1, Sexp *s2, int line)
{
    thesexp.pair.pcar = s1;
    thesexp.pair.pcdr = s2;
    type = 1;
    linenum = line;
}

Sexp::~Sexp()
{
    if(type == 1) {
	delete thesexp.pair.pcar;
	delete thesexp.pair.pcdr;
    }
}

Sexp* Sexp::car() const
{
    ASSERT(is_pair(), "tried to take car of non-list");
    return thesexp.pair.pcar;
}

Sexp* Sexp::cdr() const
{
    ASSERT(is_pair(), "tried to take cdr of non-list");
    return thesexp.pair.pcdr;
}

Dstring* Sexp::key() const
{
    ASSERT(!is_pair(), "tried to take keyword of list");
    return thesexp.key;
}

bool Sexp::is_pair() const
{
    return type;
}

Sexp* Sexp::cadr() const
{
    return cdr()->car();
}

bool Sexp::is_empty() const
{
    return cdr() == NULL;
}

int Sexp::line_number() const
{
    return linenum;
}

Sexp* Sexp::setcdr(Sexp* ncdr)
{
    ASSERT(is_pair(), "tried to set cdr of non-list");
    thesexp.pair.pcdr = ncdr;
    return this;
}

Dstring* Sexp::empty_string() const
{
    ASSERT(is_empty(), "only on an empty string");
    return car()->key();
}
#endif

static void print_list(const Sexp* S, ostream& os)
{
    os << '(';
    while(!S->is_empty())
    {
      print_sexp(S->car(), os);
      if ( !S->cdr()->is_empty() )
          os << ' ';
      S = S->cdr();
    }
    os << ')';
}

void print_sexp(const Sexp* S, ostream& os)
{
    if ( S->is_pair() )
	print_list(S, os);
    else
	os << S->key();
}

ostream& operator<<(ostream& os, Sexp *S)
{
    print_sexp(S, os);
    return os;
}

Sexp* Sexp::nreverse()
{
    Sexp *point, *prev, *next;

    if (is_empty())
	return this;

    prev = this;
    point = cdr();
    next = cdr()->cdr();

    while(true){

	point->setcdr(prev);

	if ( next->cdr() == NULL ) /* is this an empty sentinel
				      (was point empty) */
	{
	    this->setcdr(next);
	    return point;
	}
	prev = point;
	point = next;
	next = next->cdr();
    }

}

Sexp* new_empty_list()
{
    return new Sexp(new Sexp(new Dstring()), NULL);
}

int Sexp::length() const
{
    int l = 0;
    const Sexp *s = this;

    while (!s->is_empty()) {
	s = s->cdr();
	l += 1;
    }
    return l;
}
