/* -*-Mode: C;-*-
 * $Id: utils.c 1.6 Thu, 18 Jul 1996 00:40:01 -0700 jmacd $
 * utils.c:
 *
 * Copyright (C) 1996, Regents of the University of California.
 * All Rights Reserved.
 *
 * This file is part of PRCS, the Project Revision Control System.
 *
 * Authors: Paul N. Hilfinger <Hilfinger@CS.Berkeley.EDU>
 *          Josh MacDonald <jmacd@CS.Berkeley.EDU>
 */


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"

void *_malloc_(size_t size)
    /* As for malloc, but screams if storage exhausted. */
{
    char *space = malloc(size);

    if (space == NULL) {
	(void) fputs("panic: storage exhausted.\n", stderr);
	exit(2);
    }

    return (void *) space;
}

void *_mrealloc_(void *V, int size, int N, int INCR)
    /* Assuming V points to a heap-allocated vector of size N*size if	*/
    /* N>0, and INCR > 0, return a pointer (possibly == V) to an area	*/
    /* of storage containing the same data, and an additional		*/
    /* size*INCR bytes at the end.  If N is 0, then acts like		*/
    /* _malloc_, and ignores the value of V.				*/
{
    if (N == 0)
	V = malloc(size * INCR);
    else
	V = realloc(V, size * (N+INCR));
    if (V == NULL) {
	(void) fputs("panic: storage exhausted.\n", stderr);
	exit(2);
    }
    return V;
}

void *_malloc0_(size_t size)
    /* As for _malloc_, but zeroes storage. */
{
    char *space = calloc(size, 1);
    if (space == NULL) {
	(void) fputs("panic: storage exhausted.\n", stderr);
	exit(2);
    }
    return (void *) space;
}

void *_mrealloc0_(void *V, int size, int N, int INCR)
     /* As for _mrealloc_, but zeroes added storage. */
{
    if (N == 0)
	V = malloc(size * INCR);
    else
	V = realloc(V, size * (N+INCR));
    if (V == NULL) {
	(void) fputs("panic: storage exhausted.\n", stderr);
	exit(2);
    }
    memset((char *) V + N * size, 0, INCR * size);
    return V;
}
